import sys, os
import re

from PyQt5.QtWidgets import QDateTimeEdit
from PyQt5.QtCore    import QDateTime

import matplotlib
matplotlib.use('Qt5Agg')

import matplotlib.pyplot as plt
from matplotlib.ticker import (MultipleLocator, LinearLocator, IndexLocator, FormatStrFormatter, AutoMinorLocator)

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg, NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

import numpy as np
#import pandas as pd # could be useful?

import iws_defs
import json


if sys.version_info[0] < 3:
   from ConfigParser import RawConfigParser
else:
   from configparser import RawConfigParser


# importing a lot of junk which I don't need...
from PyQt5.QtWidgets import QApplication, QWidget, QMainWindow, QLabel, QGridLayout, QPushButton, QVBoxLayout, QFrame, QScrollArea, QFileDialog
from PyQt5.QtWidgets import QMessageBox, QAction, QHBoxLayout, QTabWidget, QGroupBox, QRadioButton, QListWidget, QSpacerItem, QSizePolicy
from PyQt5.QtCore    import Qt, QRect
from PyQt5.QtGui     import QPalette, QColor, QFont
from PyQt5           import QtWidgets

import iws_defs

import TimeRangeEchoReader

# Floating point number regex
fpn = re.compile('[+-]?(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?')


rates = {'rt_slow':iws_defs.rt_slow,'st_slow':iws_defs.st_slow,'rt_fast':iws_defs.rt_fast,'rt_superfast':iws_defs.rt_superfast,'st_superfast':iws_defs.st_superfast}

class MplCanvas(FigureCanvasQTAgg):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        self.fig = fig
        super(MplCanvas, self).__init__(fig)


class PlaybackWindow(QMainWindow):
#    def __init__(self, title, *args, **kwargs):
#        super(PlaybackWindow, self).__init__(*args, **kwargs)
    def __init__(self, title, config_file, src_file, start_datetime, end_datetime, config_section):
        super(PlaybackWindow, self).__init__()

        self.start_datetime = start_datetime.lstrip()
        self.end_datetime = end_datetime.lstrip().rstrip('\n')

        print(f"start_datetime = [{self.start_datetime}]")
        print(f"end_datetime   = [{self.end_datetime}]")

        self.setWindowTitle(title)
        self.name = 'HK Playback Window'
        self.setGeometry(0, 0, 1100, 600)

        self.statusbar = QtWidgets.QStatusBar(self)
        self.statusbar.setObjectName("statusbar")
        self.statusbar.showMessage('Statusbar')
        self.setStatusBar(self.statusbar)

        self.source_box = QtWidgets.QLabel('')
        self.statusbar.reformat()
#        self.statusbar.addPermanentWidget(self.message_box)
        self.statusbar.addPermanentWidget(self.source_box)
        self.setStatusBar(self.statusbar)

        widget = QWidget()
        self.scroll = QScrollArea()
        self.scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
#        self.scroll.setWidgetResizable(True)
        vbox = QVBoxLayout()

        self.open_button = QPushButton("Playback HKTM")
        self.open_button.setEnabled(True)
        self.open_button.clicked.connect(self.read_hk)

        quit_button = QPushButton("Quit")
        quit_button.clicked.connect(self.quit)

        # Indexed by iws_defs.rt_slow (2) ... iws_defs.st_superfast (6)
        # First 2 arrays unused
        self.packets_time_array = [[],[],[],[],[],[],[]]

        self.parameter_dict = {}
        self.groups = {}

        self.group_list = []

        self.plot_error = False

        loaded = self.get_config(config_file)

        if loaded == False:
            sys.exit(1)

        # Labels are keys for self.groups dictionary. Must be the same number of both.
#        self.group_list = ['VIS_ROE_BIAS_DD','RPSU_3V3VD_V_Mon', 'RPSU_VCCD_I_Mon', 'FPGA_10VA_P_V', 'VIS_CDPU_MPS_PWR', 'junk']
#        self.populate_groups()
        self.populate_parameter_dict()

###        self.reader = TimeRangeEchoReader.TimeRangeEchoReader()
        self.reader = TimeRangeEchoReader.TimeRangeEchoReader(config_section)
#        self.source = './CEA_Day3_3.25.csv'
        self.source = src_file
        self.source_box.setText('Source file: ' + self.source)

        matplotlib.rc('font',size=7)

        self.plots = []
        for i in range(len(self.group_list)):
            sc = MplCanvas(self, width=10, height=4, dpi=100)
            toolbar = NavigationToolbar(sc, self)
            self.plots.append((toolbar, sc))

        self.frame = QFrame()
        layout = QVBoxLayout()

        launch_box = QGroupBox()
        launch_box_layout = QHBoxLayout()

        self.message_box = QtWidgets.QLabel('')

#        self.start_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        self.start_time_box = QDateTimeEdit(QDateTime.fromString(self.start_datetime, "yyyy-MM-ddTHH:mm:ss.zzz"))
        self.start_time_box.setDisplayFormat("yyyy-MM-ddTHH:mm:ss.zzz")
#        self.end_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        self.end_time_box = QDateTimeEdit(QDateTime.fromString(self.end_datetime, "yyyy-MM-ddTHH:mm:ss.zzz"))
        self.end_time_box.setDisplayFormat("yyyy-MM-ddTHH:mm:ss.zzz")

        launch_box_layout.addWidget(QLabel('From:'))
        launch_box_layout.addWidget(self.start_time_box)
        launch_box_layout.addWidget(QLabel('To:'))
        launch_box_layout.addWidget(self.end_time_box)

        launch_box_layout.addWidget(self.open_button)
        launch_box_layout.addWidget(self.message_box)
        launch_box_layout.addStretch(1)
        launch_box_layout.addWidget(quit_button)
        launch_box.setLayout(launch_box_layout)

        for i in range(len(self.group_list)):
            (t,s) = self.plots[i]
            vbox.addWidget(t)
            vbox.addWidget(s)

        widget.setLayout(vbox)
        self.scroll.setWidget(widget)

        layout.addWidget(launch_box)
        layout.addWidget(self.scroll)

        self.frame.setLayout(layout)
        self.setCentralWidget(self.frame)
        # Instantiate the configuration file parser
        config = RawConfigParser()
        # Make it case-sensitive for parameter names
        config.optionxform = str
        # Open the configuration file
        config.read("HKPlayBackConfig.ini")
####        star_file_filename = config.get( config_section, "ID")



#    def get_config(self):
    def get_config(self, filename):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        #path = os.environ['HOME']
        path = iws_defs.lister_directory
#        filename, _ = QFileDialog.getOpenFileName(self, "Select a plot configuration file", path, suffixes, options=options)
        if filename:
            with open(filename) as f:
                data = json.load(f)
#                print(f"{data[0]}")
                for item in data:
                    print(f"{item['parent']}")
                    print(f"{item['unit']}")
                    print(f"{item['rate']}")
                    print(f"{item['params']}")
                    parent = item['parent']
                    self.group_list.append(parent)
                    pdict = {}
                    pdict['Unit'] = item['unit']
                    pdict['Rate'] = rates[item['rate']]
                    pdict['Params'] = item['params']
                    self.groups[parent] = pdict
#                print("group_list")
#                print(f"{self.group_list}")
#                print("groups")
#                print(f"{self.groups}")
            self.statusbar.showMessage('Using plot config file: ' + filename)
            return True
        else:
            return False


    def populate_groups(self):
        #self.groups['VSGT2004'] = {'Unit':'Volts', 'Rate':iws_defs.st_slow, 'Params':['VS1T2004', 'VS2T2004', 'VS3T2004', 'VS4T2004', 'VS5T2004', 'VS6T2004']}
        self.groups['VIS_ROE_BIAS_DD']  = {'Unit':'Volts',
                                           'Rate':iws_defs.st_slow,
                                           'Params':self.make_param_name_array('VS%XT2004', 12)}
        self.groups['RPSU_3V3VD_V_Mon'] = {'Unit':'Volts', 'Rate':iws_defs.rt_superfast, 'Params':self.make_param_name_array('VP%XT2512', 2)}
        self.groups['RPSU_VCCD_I_Mon']  = {'Unit':'Amps',  'Rate':iws_defs.rt_fast,      'Params':self.make_param_name_array('VP%XT2503', 2)}
        self.groups['FPGA_10VA_P_V']    = {'Unit':'Amps',  'Rate':iws_defs.rt_slow,      'Params':self.make_param_name_array('VS%XT2009', 12)}

        # Enumerated parameter test
        self.groups['VIS_CDPU_MPS_PWR'] = {'Unit':'', 'Rate':iws_defs.rt_slow, 'Params':['VDGT2731','VDGT2732','VDGT2733']}

        # Mix of parameters, np.nan test
        self.groups['junk'] = {'Unit':'', 'Rate':iws_defs.st_slow, 'Params':['VS1T2078','VS1T2056','VS7T2079']}


    def make_param_name_array(self, name, times):
        arr = []
        for i in range(times):
#            print(f"{name}" % (i+1))
            arr.append(f"{name}" % (i+1))
#        print(arr)
        return arr


    def populate_parameter_dict(self):
        for group in self.group_list:
            for param in self.groups[group]['Params']:
                self.parameter_dict[param] = []


    def add_parameter(self, mib_id, value):
        try:
            if value == '*NO VAL*':
                value = np.nan
            elif fpn.match(value):
                value = float("%3.03f" % float(value))
            self.parameter_dict[mib_id].append(value)
        except Exception as e:
            #print(f"add_parameter exception {e}")
            pass


    def clear_parameters(self):
        for p in self.parameter_dict.keys():
            self.parameter_dict[p] = []
        self.packets_time_array = [[],[],[],[],[],[],[]]


    def quit(self):
        quitButtonReply = QtWidgets.QMessageBox.question(self,
                                                         'PyQt5 message', 'Are you sure you want to quit the program?',
                                                         QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)
        if quitButtonReply == QtWidgets.QMessageBox.Yes:
            self.close()
            exit(1)


    def read_hk(self):
       print("read_hk")
       self.message_box.setText('Reading file...')
       QtWidgets.QApplication.processEvents()
       
       self.reader.reset()
       self.reader.open_csv_file(self.source, 'playback')
       
       st = self.start_time_box.dateTime().toString("yyyy-MM-ddTHH:mm:ss.zzz")
       self.reader.setStartDateTime(st)
       et = self.end_time_box.dateTime().toString("yyyy-MM-ddTHH:mm:ss.zzz")
       self.reader.setEndDateTime(et)
       
       # For testing with CEA_Day3_3.25.csv
       # First packet obt: 2019-11-29T07:22:11.751
       # Last packet obt:  2019-11-29T09:08:57.190
       
        #self.reader.setStartDateTime('2019-11-29T08:09:14.000')
        #        self.reader.setStartDateTime('2019-11-29T07:22:14.000')
       
        #self.reader.setEndDateTime(  '2019-11-29T08:00:00.000')
        #        self.reader.setEndDateTime('2019-11-29T09:07:58.000')
       
        #self.reader.setEndDateTime('2019-11-29T09:09:58.000') # End date time past obt time of last packet in CEA file
       
       self.clear_parameters()
#        return
       
       while self.reader.data_available():
          result = self.reader.next_line()
          
          if result != 0:
             print('cvs_update: bad read')
             # which do I do?
                #break
                #return
             
          self.record()
             
       self.reader.close_input()
       self.plot_error = False
       self.message_box.setText('Creating plots...')
       QtWidgets.QApplication.processEvents()
       self.do_plots()
       #        print(f"st = {st}")
       #        print(f"et = {et}")
       if self.plot_error == False:
          self.message_box.setText('')
#        QtWidgets.QApplication.processEvents()


    def record(self):
        (line_type, string) = self.reader.decode_csv_line()

        if line_type == TimeRangeEchoReader.LineType.Event:
#            print("EVENT: " + string)
            pass

        elif line_type == TimeRangeEchoReader.LineType.TC:
#            print("TC: " + string)
            pass

        elif line_type == TimeRangeEchoReader.LineType.TM:
            if self.reader.is_in_range():
                mib_id = string[1]
                val = string[3]
                self.add_parameter(mib_id, val)
                pass

        ###elif line_type == TimeRangeEchoReader.LineType.TM_PACKET:
        elif line_type == TimeRangeEchoReader.LineType.HK_PACKET:
            # Has already updated self._logTime with obt from packet
#            print("TM: " + string[0] + ' ' + string[1] + ' ' + string[2] + ' ' + string[3])
            if self.reader.is_in_range():
                sid = int(string[3])
                self.packets_time_array[sid].append(string[0])

        elif line_type == TimeRangeEchoReader.LineType.Empty:
            print("Empty line")
            pass

        else:
            pass


    def do_plots(self):
        for i in range(len(self.group_list)):
            self.do_plot(self.plots[i], self.group_list[i], self.groups[self.group_list[i]])


    # If the size of the time array does not match the size of the data array then bye, bye
    def do_plot(self, g, title, group):
        (t, graph) = g
        graph.axes.cla()
        rate = group['Rate']
        l = len(self.packets_time_array[rate])
        if l == 0:
            return

        # x tick label spacing
        d = int(l * 0.05)
        if d > 60:
            d = 60

        n = int(l / d)
        print(f"rate, l, d, n: {rate}, {l}, {d}, {n}")
        graph.axes.xaxis.set_major_locator(IndexLocator(n, 0))

        #try:
        for parameter in group['Params']:
           self.message_box.setText(f"Plotting {title} {parameter}")
           l1 = len(self.parameter_dict[parameter])
           print(f"l, l1 = {l}, {l1} ({parameter})")
           if l != l1:
              self.message_box.setText(f"Error plotting {parameter}")
              QtWidgets.QApplication.processEvents()
              return

                # Check here for no data (even though might have a populated time array).
                # For now could break but a gui indicator would be useful
#                print(self.parameter_dict[parameter])
           graph.axes.plot(self.packets_time_array[rate], self.parameter_dict[parameter])
        #try:
           print("Fixing title")
           # graph.fig.suptitle("suptitle", x=0.1)
           graph.axes.set_title(title)
           graph.axes.set_ylabel(group['Unit'])
           graph.axes.set_xlabel('Date Time')
           graph.axes.grid()
           # graph.axes.set_xticklabels(self.packets_time_array[rate][slice(1, l, n)], rotation = 45, ha = 'right')
        try:
           print(len(self.packets_time_array[rate][slice(0, l, n)]))
           graph.axes.set_xticklabels(self.packets_time_array[rate][slice(0, l, n)], rotation = 45, ha = 'right')
           graph.fig.tight_layout()
           graph.axes.legend(group['Params'])
           graph.draw()
        except Exception as e:
           self.plot_error = True
           self.message_box.setText(f"Error in plotting {title} {parameter}: " + str(e))
           print(f"rate, l, d, n: {rate}, {l}, {d}, {n}")
           QtWidgets.QApplication.processEvents()


    """
    # If the size of the time array does not match the size of the data array then bye, bye
    def do_plot(self, g, title, group):
        (t, graph) = g
        graph.axes.cla()
        rate = group['Rate']
        l = len(self.packets_time_array[rate])
        if l == 0:
            return

        # x tick label spacing
        d = int(l * 0.05)
        if d > 60:
            d = 60

        n = int(l / d)
        print(f"rate, l, d, n: {rate}, {l}, {d}, {n}")
        graph.axes.xaxis.set_major_locator(IndexLocator(n, 0))

        try:
            for parameter in group['Params']:
                l1 = len(self.parameter_dict[parameter])
                print(f"l, l1 = {l}, {l1} ({parameter})")
                if l != l1:
                    self.message_box.setText(f"Error plotting {parameter}")
                    QtWidgets.QApplication.processEvents()
                    return

                # Check here for no data (even though might have a populated time array).
                # For now could break but a gui indicator would be useful
#                print(self.parameter_dict[parameter])
                graph.axes.plot(self.packets_time_array[rate], self.parameter_dict[parameter])

            print("Fixing title")
            #        graph.fig.suptitle("suptitle", x=0.1)
            graph.axes.set_title(title)
            graph.axes.set_ylabel(group['Unit'])
            graph.axes.set_xlabel('Date Time')
            graph.axes.grid()
            #        graph.axes.set_xticklabels(self.packets_time_array[rate][slice(1, l, n)], rotation = 45, ha = 'right')
            print(len(self.packets_time_array[rate][slice(0, l, n)]))
            graph.axes.set_xticklabels(self.packets_time_array[rate][slice(0, l, n)], rotation = 45, ha = 'right')
            graph.fig.tight_layout()
            graph.axes.legend(group['Params'])
            graph.draw()
        except Exception as e:
            self.plot_error = True
            self.message_box.setText(f"Error in plotting {title}: " + str(e))
            QtWidgets.QApplication.processEvents()
            """


if __name__ == '__main__':
    app = QApplication(sys.argv)
    #app.setStyleSheet("QGroupBox {background-color:pink;}")

    print(f"Number of args = {len(sys.argv)}")
    print(f"Arg 0 : {sys.argv[0]}")
    print(f"Arg 1 : {sys.argv[1]}")   # config_file
    print(f"Arg 2 : {sys.argv[2]}")   # src_file
    print(f"Arg 3 : {sys.argv[3]}")   # start_datetime
    print(f"Arg 4 : {sys.argv[4]}")   # end_datetime
#    print(f"Arg 5 : {sys.argv[5]}")

###    window = PlaybackWindow("MainWindow", sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    window = PlaybackWindow("MainWindow", sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], 'CEA_ORIG')
    window.show()

    sys.exit(app.exec_())
