from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import Qt, QMimeData, QRegExp
from PyQt5.QtWidgets import QTreeWidgetItem, QFrame, QFileDialog, QMessageBox
from PyQt5.QtGui import QDrag, QFont, QRegExpValidator

import json

import make_config_defs

from lib.Lists.Lists import List, ListItem
#import List
#import ListItem

from lib.Config import PlotFrame

"""
This class mimics the plot frame in the HKTM monitor and is used to define a new plot window for the monitor.
"""
class MPlotFrame(PlotFrame.PlotFrame):

    def __init__(self, parent=None):
        super(MPlotFrame, self).__init__(parent, subClass=self)

        print(f"Layout = {self.layout()}")


    def make_list_frame(self):
        frame = QtWidgets.QFrame()
        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)

        # DEBUG
        #frame.setStyleSheet("background-color:green;")

        frame.setMaximumWidth(250)
        layout = QtWidgets.QVBoxLayout()
        #frame.setContentsMargins(0, 0, 0, 0)

        tabs_list_frame = self.make_tab_list_frame()
        control_frame = self.make_control_frame()

        layout.addWidget(tabs_list_frame)
        layout.addWidget(control_frame)
        frame.setLayout(layout)
        return frame


    def make_tab_list_frame(self):
        frame = QtWidgets.QGroupBox("Tabs")
        layout = QtWidgets.QVBoxLayout()

        tabs_list = QtWidgets.QListWidget()
        tabs_list.setObjectName('tabs_list')
#        tabs_list.setMaximumWidth(200)
        self.tabs_list = tabs_list

        layout.addWidget(tabs_list)
        frame.setLayout(layout)
        return frame


    def make_control_frame(self):
        frame = QtWidgets.QFrame()
        layout = QtWidgets.QVBoxLayout()

        name_frame = self.make_name_frame()
        self.add_def_button = QtWidgets.QPushButton('Add')
#        self.add_def_button.setMaximumWidth(100)

        layout.addWidget(name_frame)
        layout.addWidget(self.add_def_button)
        frame.setLayout(layout)
        return frame


    def make_name_frame(self):
        frame = QtWidgets.QFrame()
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Name:")
        self.add_def_box = QtWidgets.QLineEdit()
#        self.add_def_box.setMaximumWidth(100)
        layout.addWidget(label)
        layout.addWidget(self.add_def_box, QtCore.Qt.AlignLeft)
        frame.setLayout(layout)
        return frame


    """
    def make_label_frame(self):
        frame  = QtWidgets.QFrame()
        layout = QtWidgets.QHBoxLayout()

        return frame


    def handleActivated(self, index):
        self.combo_index = index


    def clear(self):
        self.page_name_box.setText('')
        self.min_box.setText('')
        self.max_box.setText('')
        nitems = self.list.count()
        for n in range(nitems):
            #print(f"Removing item {n}")
            self.list.takeItem(0)

    def nameEdited(self):
        name = self.page_name_box.text()
        enable = True if name != '' else False
        self.save_button.setEnabled(enable)
        self.add_button.setEnabled(enable)
        pass


    def new_pressed(self):
        pass


    def add_pressed(self):
        name = self.page_name_box.text()
        if name == '':
            self.parent.statusbar.showMessage("Plot name is required")
            return False
        unit = self.combo.itemText(self.combo_index)
        minv = self.min_box.text()
        if minv == '':
            self.parent.statusbar.showMessage("Minimum value is required")
            return False
        maxv = self.max_box.text()
        if maxv == '':
            self.parent.statusbar.showMessage("Maximum value is required")
            return False
        if self.list.count() == 0:
            self.parent.statusbar.showMessage("At least one plot parameter is required")
            return False
        ret = self.parent.add_plot(name, self.list, minv, maxv, unit)
        if ret:
            self.clear()
            return ret


    def save(self):
        name = self.page_name_box.text()
        unit = self.combo.itemText(self.combo_index)
        minv = self.min_box.text()
        maxv = self.max_box.text()
        min_value = 0.0
        max_value = 0.0
        try:
            min_value = float(minv)
            max_value = float(maxv)
        except Exception as e:
            print(f"An error occurred converting minv {minv} or maxv {maxv}")
        # Check name is set
        if self.list.count() == 0:
            self.parent.statusbar.showMessage("At least one parameter is required")
        else:
            dict = {}
            group = []
            group.append(self.parent.make_group(self.list, 0))
            dict['name'] = name
            dict['naxes'] = 1
            yaxis1 = {}
            yaxis1['label'] = unit
            yaxis1['min'] = min_value
            yaxis1['max'] = max_value
            dict['yaxis1'] = yaxis1
            dict['yaxis2'] = {}
            dict['labels'] = group
            
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            suffixes = 'json file (*.json)'

            filename, _ = QFileDialog.getSaveFileName(self, "QFileDialog.getSaveFileName()", make_config_defs.plot_tabs_dir, suffixes, options=options)
            if filename:
                print(f"filename is {filename}")
                if filename.endswith('.json'):
                    print(f"(has json) filename = {filename}")
                    pass
                else:
                    filename += '.json'
                    print(f"(needs json) filename = {filename}")
                with open(filename, 'w') as outfile:
                    json.dump(dict, outfile)
        pass


    def read(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", make_config_defs.plot_tabs_dir, suffixes, options=options)
        if filename:
            self.clear()
            with open(filename) as f:
                data = json.load(f)
                print(f"MPlotFrame : read : {data}")
                self.page_name_box.setText(data['name'])
                yaxis1 = data['yaxis1']
                self.min_box.setText(str(yaxis1['min']))
                self.max_box.setText(str(yaxis1['max']))
                unit = yaxis1['label']
                index = self.combo.findText(unit)
                if index == -1:
                    print("MPlotFrame : read : ARRRGHHH!")
                else:
                    self.combo.setCurrentIndex(index)
                labels = data['labels']
                group = labels[0] # [{'row':0,..., 'labels':[]}]
                labels = group['labels']
                for label in labels:
                    item = ListItem(label['mib_name'], label['mib_id'], label['source'], label['unit'], 0)
                    self.list.addItem(item)
        pass


    def populate_group(self, data):
        print(f"populate_group: {data}")
        self.page_name_box.setText(data['name'])
        yaxis1 = data['yaxis1']
        self.min_box.setText(str(yaxis1['min']))
        self.max_box.setText(str(yaxis1['max']))
        unit = yaxis1['label']
        index = self.combo.findText(unit)
        # DEBUG
        print(f"Name {self.page_name_box.text()}")
        print(f"Unit {unit}")
        if index == -1:
                print("MPlotFrame : populate_group : ARRRGHHH!")
        else:
            self.combo.setCurrentIndex(index)
            labels = data['labels']
            group = labels[0] # [{'row':0,..., 'labels':[]}]

#        labels = group['labels']

            for label in labels:
                item = ListItem(label['mib_name'], label['mib_id'], label['source'], label['unit'], 0)
                self.list.addItem(item)


    def set_selected_box(self, name):
        pass
        """
